/**
 * MicroJSON library. 
 * 
 * Provides JSON structures parsing and generating in C++ way. 
 * 
 * Distributed under GPL v3 
 * 
 * Author: 
 *      Grigory Gorelov (gorelov@grigory.info)
 *      See more information on grigory.info
 */

#include <unistd.h>

#include "Node.h"

MicroJSON::Node & MicroJSON::Node::operator=(const Node &Original) {

    // Check if objects are not same
    if ((unsigned int long) &Original != (unsigned int long) this) {
        Type = Original.Type;

        Name = Original.Name;

        NumberValue = Original.NumberValue;
        StringValue = Original.StringValue;
        BoolValue = Original.BoolValue;

        for (unsigned int i = 0; i < Original.Children.size(); i++) {
            Node *NewNode = new Node(*Original.Children[i]);
            Children.push_back(NewNode);
        }
    }

    return *this;

}

MicroJSON::Node::Node(const Node &Original) {
    Type = Original.Type;

    Name = Original.Name;

    NumberValue = Original.NumberValue;
    StringValue = Original.StringValue;
    BoolValue = Original.BoolValue;

    for (unsigned int i = 0; i < Original.Children.size(); i++) {
        Node *NewNode = new Node(*Original.Children[i]);
        Children.push_back(NewNode);
    }
}

bool MicroJSON::Node::IsObject() const {
    return Type == MicroJSON::Types::Object;
}

bool MicroJSON::Node::IsNumber() const {
    return Type == MicroJSON::Types::Number;
}

bool MicroJSON::Node::IsString() const {
    return Type == MicroJSON::Types::String;
}

bool MicroJSON::Node::IsBool() const {
    return Type == MicroJSON::Types::Bool;
}

bool MicroJSON::Node::IsArray() const {
    return Type == MicroJSON::Types::Array;
}

bool MicroJSON::Node::IsNull() const {
    return Type == MicroJSON::Types::Null;
}

MicroJSON::Node * MicroJSON::Node::GetSubNode(const UTF8::String &Name) const {
    if (IsObject()) {
        for (int i = 0; i < Children.size(); i++) {
            if (Children[i]->GetName() == Name) {
                return Children[i];
            }
        }
        return NULL;
    } else {
        throw Exception("Trying to get subobject but parent type is not «Object»");
    }
}

std::vector <MicroJSON::Node *> MicroJSON::Node::GetChildren() const {
    if (IsObject() || IsArray()) {
        return Children;
    } else {
        throw Exception("Trying to get children but the type is not «Array» or «Object»");
    }
}

long double MicroJSON::Node::GetNumber() const {
    if (IsNumber()) {
        return NumberValue;
    } else {
        throw Exception("Trying to get number value but the type is not «Number»");
    }
}

UTF8::String MicroJSON::Node::GetString() const {
    if (IsString()) {
        return StringValue;
    } else {
        throw Exception("Trying to get string value but the type is not «String»");
    }
}

bool MicroJSON::Node::GetBool() const {
    if (IsBool()) {
        return BoolValue;
    } else {
        throw Exception("Trying to get bool value but the type is not «Bool»");
    }
}

int MicroJSON::Node::GetType() const {
    return Type;
}

MicroJSON::Node::Node() {
    Type = Types::Object;
}

MicroJSON::Node::Node(const int Type) {
    this->Type = Type;
}

MicroJSON::Node::~Node() {
    for (int i = 0; i < Children.size(); i++) {
        delete Children[i];
    }
}

bool MicroJSON::Node::_Parse_CharacterIsWhitespace(const UTF8::String &Char) {
    if ((Char == "\n") || (Char == "\r") || (Char == "\t") || (Char == " ")) {
        return true;
    } else {
        return false;
    }
}

void MicroJSON::Node::Parse(const UTF8::String &JSONString) {

    if (Children.size() > 0) {
        throw Exception("The Node already has children. Will not parse JSON string into it. The string was: «" + JSONString + "»");
    }

    unsigned int i = 0;

    while (i < JSONString.Length()) {
        if (JSONString[i] == "{") {
            _Parse_ParseObject(this, JSONString, i);

            // We are on the top of hierarchy and object parse is finished. The job is done.
            return;
        } else if (!_Parse_CharacterIsWhitespace(JSONString[i])) {
            if (i == 0) {
                throw Exception("Cannot parse JSON string: " + JSONString);
            } else {
                throw Exception("Cannot parse JSON string (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
            }
        }
        i++;
    }


    // If we are here it means JSON string ended even not started.
    throw Exception("JSON string suddenly ended (while parsing top level): «" + JSONString + "\"");
}

void MicroJSON::Node::_Parse_ParseValue(MicroJSON::Node *Object, const UTF8::String &JSONString, unsigned int &i) {
    const unsigned int Status_WaitingForValue = 1;
    const unsigned int Status_AccumulatingNumberValue = 2;
    const unsigned int Status_AccumulatingStringValue = 3;
    const unsigned int Status_AccumulatingStringValue_AfterSlash = 4;
    const unsigned int Status_AccumulatingBoolValue_True = 5;
    const unsigned int Status_AccumulatingBoolValue_False = 6;
    const unsigned int Status_AccumulatingNullValue = 7;


    unsigned int Status = Status_WaitingForValue;
    UTF8::String Value;


    while (i < JSONString.Length()) {
        UTF8::String Char = JSONString[i];

        switch (Status) {

            case Status_WaitingForValue:
                // So we are after colon or bracket and waiting for a value. We don't know type yet.
                if (Char.CharacterIsOneOfThese("1234567890.-")) {
                    // Value is a number. Accumulating it.
                    Status = Status_AccumulatingNumberValue;
                    Value += Char;
                } else {
                    if (Char == "\"") {
                        Status = Status_AccumulatingStringValue;
                    } else {
                        if (Char.CharacterIsOneOfThese("Tt")) {
                            Status = Status_AccumulatingBoolValue_True;
                            Value += "t";
                        } else {
                            if (Char.CharacterIsOneOfThese("Ff")) {
                                Status = Status_AccumulatingBoolValue_False;
                                Value += "f";
                            } else {
                                if (Char.CharacterIsOneOfThese("Nn")) {
                                    Status = Status_AccumulatingNullValue;
                                    Value += "n";
                                } else {
                                    if (Char == "{") {
                                        _Parse_ParseObject(Object, JSONString, i);

                                        // Before _Parse_ParseObject exit it increments "i" value.
                                        // But after ParseValue exited "i" will be incremented again. Solving it by i--;
                                        i--;
                                        return;
                                    } else {
                                        if (Char == "[") {
                                            Object->SetType(Types::Array);
                                            _Parse_ParseArray(Object, JSONString, i);
                                            return;
                                        } else {
                                            if (!_Parse_CharacterIsWhitespace(Char)) {
                                                throw Exception("Number, object, string or array expected here. But it is not. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                break;

            case Status_AccumulatingBoolValue_False:
                // Bool true value detected. Accumulate and confirm it.
                if (Char == "," || Char == "}" || Char == "]" || _Parse_CharacterIsWhitespace(Char)) {
                    // Finished boolean "false" value collecting here.
                    if (Value == "false") {
                        Object->SetType(Types::Bool);
                        Object->SetBool(false);
                    } else {
                        throw Exception("Error parsing «false» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                    }

                    // We can be on "}" or "," position. Caller must stay on it.
                    // Need to decrement to stay on potentially "}" or "," character.
                    i--;

                    return;
                } else {
                    if (Char.CharacterIsOneOfThese("Aa")) {
                        if (Value == "f") {
                            Value += "a";
                        } else {
                            throw Exception("Symbol «" + Char + "» cannot be in this position in «false» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                        }
                    } else {
                        if (Char.CharacterIsOneOfThese("Ll")) {
                            if (Value == "fa") {
                                Value += "l";
                            } else {
                                throw Exception("Symbol «" + Char + "» cannot be in this position in «false» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                            }
                        } else {
                            if (Char.CharacterIsOneOfThese("Ss")) {
                                if (Value == "fal") {
                                    Value += "s";
                                } else {
                                    throw Exception("Symbol «" + Char + "» cannot be in this position in «false» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                                }
                            } else {
                                if (Char.CharacterIsOneOfThese("Ee")) {
                                    if (Value == "fals") {
                                        Value += "e";
                                    } else {
                                        throw Exception("Symbol «" + Char + "» cannot be in this position in «false» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                                    }
                                } else {
                                    throw Exception("Symbol «" + Char + "» cannot be in this position in «false» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                                }
                            }
                        }

                    }
                }
                break;

            case Status_AccumulatingBoolValue_True:
                // Bool true value detected. Accumulate and confirm it.
                if (Char == "," || Char == "}" || Char == "]" || _Parse_CharacterIsWhitespace(Char)) {
                    // Finished boolean "true" value collecting here.
                    if (Value == "true") {
                        Object->SetType(Types::Bool);
                        Object->SetBool(true);
                    } else {
                        throw Exception("Error parsing «true» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                    }

                    // We can be on "}" or "," position. Caller must stay on it.
                    // Need to decrement to stay on potentially "}" or "," character.
                    i--;

                    return;
                } else {
                    if (Char.CharacterIsOneOfThese("Rr")) {
                        if (Value == "t") {
                            Value += "r";
                        } else {
                            throw Exception("Symbol «" + Char + "» cannot be in this position in «true» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                        }
                    } else {
                        if (Char.CharacterIsOneOfThese("Uu")) {
                            if (Value == "tr") {
                                Value += "u";
                            } else {
                                throw Exception("Symbol «" + Char + "» cannot be in this position in «true» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                            }
                        } else {
                            if (Char.CharacterIsOneOfThese("Ee")) {
                                if (Value == "tru") {
                                    Value += "e";
                                } else {
                                    throw Exception("Symbol «" + Char + "» cannot be in this position in «true» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                                }
                            } else {
                                throw Exception("Symbol «" + Char + "» cannot be in this position in «true» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                            }
                        }
                    }

                }
                break;


            case Status_AccumulatingNullValue:
                // Null value detected. Accumulate and confirm it.
                if (Char == "," || Char == "}" || Char == "]" || _Parse_CharacterIsWhitespace(Char)) {
                    // Finished "null" value collecting here.
                    if (Value == "null") {
                        Object->SetType(Types::Null);
                    } else {
                        throw Exception("Error parsing «null» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                    }

                    // We can be on "}" or "," position. Caller must stay on it.
                    // Need to decrement to stay on potentially "}" or "," character.
                    i--;

                    return;
                } else {
                    if (Char.CharacterIsOneOfThese("Uu")) {
                        if (Value == "n") {
                            Value += "u";
                        } else {
                            throw Exception("Symbol «" + Char + "» cannot be in this position in «null» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                        }
                    } else {
                        if (Char.CharacterIsOneOfThese("Ll")) {
                            if ((Value == "nu") || (Value == "nul")) {
                                Value += "l";
                            } else {
                                throw Exception("Symbol «" + Char + "» cannot be in this position in «null» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                            }
                        } else {
                            throw Exception("Symbol «" + Char + "» cannot be in this position in «null» value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                        }
                    }

                }
                break;

            case Status_AccumulatingStringValue_AfterSlash:
                // Collecting string we have found slash. That means second slash, "n", "r" or
                // quotes must be next. If not it is error.
                if (Char.CharacterIsOneOfThese("\"\\nrNR")) {
                    Status = Status_AccumulatingStringValue;

                    if (Char.CharacterIsOneOfThese("nN")) {
                        Value += "\n";
                    } else if (Char.CharacterIsOneOfThese("rR")) {
                        Value += "\r";
                    } else if (Char.CharacterIsOneOfThese("\"")) {
                        Value += "\"";
                    } else if (Char.CharacterIsOneOfThese("\\")) {
                        Value += "\\";
                    }
                } else {
                    throw Exception("Slash must be followed by second slash, «n», «r» or quotes in string value. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                }
                break;

            case Status_AccumulatingStringValue:
                // It was detected that value is string. Collecting it. 
                // Don't foget about slash.
                if (Char == "\"") {
                    // Finished string collecting here.

                    Object->SetType(Types::String);
                    Object->SetString(Value);

                    return;
                } else {
                    if (Char == "\\") {
                        Status = Status_AccumulatingStringValue_AfterSlash;
                    } else {
                        Value += Char;
                    }
                }

                break;

            case Status_AccumulatingNumberValue:
                // We have number as value. Accumulating it.
                if (Char.CharacterIsOneOfThese("1234567890.")) {
                    // Value is a number and we are collecting it. Not that it cannot contain "-" now.
                    if (Char == ".") {
                        if (Value.HasThisString(".")) {
                            throw Exception("Numeric value cannot contain two dots. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                        }
                    }
                    Value += Char;
                } else {
                    if (Char == "," || Char == "}" || Char == "]" || _Parse_CharacterIsWhitespace(Char)) {
                        // Finished number collecting here.

                        Object->SetType(Types::Number);
                        Object->SetNumber(Value.ToDouble());

                        // We can be on "}", "]" or "," position. Caller must stay on it.
                        // Need to decrement to stay on potentially "}" or "," character.
                        i--;

                        return;
                    } else {
                        throw Exception("Numeric value cannot contain symbol «" + Char + "». (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                    }
                }
                break;
        }

        i++;

    }

    // If we are here it means JSON string ended unexpectedly.
    throw Exception("JSON string suddenly ended (while parsing value): «" + JSONString + "»");

}

void MicroJSON::Node::_Parse_ParseObject(MicroJSON::Node *Object, const UTF8::String &JSONString, unsigned int &i) {

    const unsigned int Status_WaitingForName = 1;
    const unsigned int Status_AccumulatingName = 2;
    const unsigned int Status_WaitingForColon = 3;
    const unsigned int Status_AfterValueParsed = 6;

    unsigned int Status = Status_WaitingForName;

    UTF8::String Name;

    // Now where are on "{" character so we need to increment i.
    i++;

    while (i < JSONString.Length()) {
        UTF8::String Char = JSONString[i];
        switch (Status) {
            case Status_WaitingForName:
                // We are in object, so expecting name -------------------------
                if (Char == "\"") { // Quotes is start of the name
                    Status = Status_AccumulatingName;
                } else {
                    if (Char == "}") { // Object is finished
                        return;
                    } else if (!_Parse_CharacterIsWhitespace(Char)) {
                        throw Exception("Cannot parse JSON string (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                    }
                }
                break;

            case Status_AccumulatingName:
                // Quotes found and now we are accumulating name characters ----

                if (Char == "\"") { // Name is ended, now colon needed
                    Status = Status_WaitingForColon;
                } else {

                    if (Char.CharacterIsOneOfThese("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_")) {
                        if ((Name.Length() == 0) && (Char.CharacterIsOneOfThese("1234567890"))) {
                            throw Exception("Field name cannot be started from number (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                        } else {
                            Name += Char;
                        }

                    } else {
                        throw Exception("Field name cannot contain this character (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                    }
                }
                break;

            case Status_WaitingForColon:
                // We have just got name. Now colon is needed.
                if (Char == ":") { // Color divides name from value
                    // Now we are on ":" symbol. Start value parsing after it.
                    i++;

                    _Parse_ParseValue(Object->AddObject(Name), JSONString, i);

                    Name.Empty();


                    Status = Status_AfterValueParsed;
                } else {
                    if (!_Parse_CharacterIsWhitespace(Char)) {
                        throw Exception("Colon expected here. But it is not. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                    }
                }
                break;



            case Status_AfterValueParsed:
                // We have just finished value parsing and now are waiting from "," or "}"
                if (Char == ",") {
                    // After comma must be name in object
                    Status = Status_WaitingForName;
                } else {
                    if (Char == "}") {
                        // Increment before return.
                        i++;

                        return;
                    } else {
                        if (!_Parse_CharacterIsWhitespace(Char)) {
                            throw Exception("Colon expected here. But it is not. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                        }
                    }
                }

                break;
        }

        i++;

    }

    // If we are here it means JSON string ended unexpectedly.
    throw Exception("JSON string suddenly ended (while parsing object): «" + JSONString + "»");


}

void MicroJSON::Node::_Parse_ParseArray(MicroJSON::Node *Object, const UTF8::String &JSONString, unsigned int &i) {

    const unsigned int Status_WaitingForValue = 1;
    const unsigned int Status_AfterValueParsed = 2;

    unsigned int Status = Status_WaitingForValue;

    UTF8::String Name;

    // Now where are on "[" character so we need to increment i.
    i++;

    while (i < JSONString.Length()) {
        UTF8::String Char = JSONString[i];
        switch (Status) {
            case Status_WaitingForValue:
                if (!_Parse_CharacterIsWhitespace(Char)) {
                    if (Char == "]") {
                        return;
                    } else {
                        _Parse_ParseValue(Object->AddObject(), JSONString, i);
                        Status = Status_AfterValueParsed;
                    }
                }
                break;

            case Status_AfterValueParsed:
                // We have just finished value parsing and now are waiting from "," or "}"
                if (Char == ",") {
                    // After comma must be name in object
                    Status = Status_WaitingForValue;
                } else {
                    if (Char == "]") {
                        // We must not increment "i" here. It will be incremeneted in
                        // ParseObject function after "while" cycle.
                        return;
                    } else {
                        if (!_Parse_CharacterIsWhitespace(Char)) {
                            throw Exception("Sqared braces are expected here. But it is not. (below string is truncated to error position): «" + JSONString.Substring(0, i + 1) + "»");
                        }
                    }
                }

                break;
        }

        i++;

    }

    // If we are here it means JSON string ended unexpectedly.
    throw Exception("JSON string suddenly ended (while parsing array): «" + JSONString + "»");


}

UTF8::String MicroJSON::Node::Render() const {

    UTF8::String Result;

    if (Type == Types::Object) {
        if (Name.Length()) {
            Result += "\"" + Name.Replace("\"", "\\\"") + "\":";
        }
        Result += "{";
        for (int i = 0; i < Children.size(); i++) {
            if (i) {
                Result += ",";
            }

            Result += Children[i]->Render();
        }
        Result += "}";
    }

    if (Type == Types::Array) {
        if (Name.Length()) {
            Result += "\"" + Name.Replace("\"", "\\\"") + "\":";
        }

        Result += "[";
        for (int i = 0; i < Children.size(); i++) {
            if (i) {
                Result += ",";
            }

            Result += Children[i]->Render();
        }
        Result += "]";
    }

    if (Type == Types::Number) {
        if (Name.Length()) {
            Result = "\"" + Name.Replace("\"", "\\\"") + "\":";
        }

        Result += NumberValue;
    }

    if (Type == Types::String) {
        if (Name.Length()) {
            Result = "\"" + Name.Replace("\"", "\\\"") + "\":";
        }

        // Escaping \, ", \n, \r,
        Result += "\"" + StringValue.Replace("\\", "\\\\").Replace("\"", "\\\"").Replace("\n", "\\n").Replace("\r", "\\r") + "\"";
    }


    if (Type == Types::Bool) {
        if (Name.Length()) {
            Result = "\"" + Name.Replace("\"", "\\\"") + "\":";
        }

        Result += (BoolValue ? "true" : "false");
    }

    if (Type == Types::Null) {
        if (Name.Length()) {
            Result = "\"" + Name.Replace("\"", "\\\"") + "\":";
        }

        Result += "null";
    }


    return Result;
}

void MicroJSON::Node::AddNumber(const UTF8::String &Name, const double Value) {
    if (Type != Types::Object) {
        throw Exception("Cannot add number due to type is not parent.");
    } else {

        Node *Child = new Node(Types::Number);

        Child->SetName(Name);
        Child->SetNumber(Value);

        Children.push_back(Child);
    }
}

void MicroJSON::Node::AddNumber(const long Value) {
    if (Type != Types::Array) {
        throw Exception("Cannot add number to array due to type is not array.");
    } else {

        Node *Child = new Node(Types::Number);

        Child->SetNumber(Value);

        Children.push_back(Child);
    }
}

void MicroJSON::Node::AddString(const UTF8::String &Name, const UTF8::String & Value) {
    if (Type != Types::Object) {
        throw Exception("Cannot add object due to type is not parent.");
    } else {

        Node *Child = new Node(Types::String);

        Child->SetName(Name);
        Child->SetString(Value);

        Children.push_back(Child);
    }
}

void MicroJSON::Node::AddString(const UTF8::String & Value) {
    if (Type != Types::Array) {
        throw Exception("Cannot add object to  array due to type is not array.");
    } else {

        Node *Child = new Node(Types::String);

        Child->SetString(Value);

        Children.push_back(Child);
    }
}

void MicroJSON::Node::AddBool(const UTF8::String &Name, const bool &Value) {
    if (Type != Types::Object) {
        throw Exception("Cannot add bool due to type is not parent.");
    } else {

        Node *Child = new Node(Types::Bool);

        Child->SetName(Name);
        Child->SetBool(Value);

        Children.push_back(Child);
    }
}

void MicroJSON::Node::AddBool(const bool &Value) {
    if (Type != Types::Array) {
        throw Exception("Cannot add bool to array due to type is not array.");
    } else {

        Node *Child = new Node(Types::Bool);

        Child->SetBool(Value);

        Children.push_back(Child);
    }
}

MicroJSON::Node * MicroJSON::Node::AddObject(const UTF8::String & Name) {
    if (Type != Types::Object) {
        throw Exception("Cannot add object due to type is not parent.");
    } else {

        Node *Child = new Node(Types::Object);

        Child->SetName(Name);

        Children.push_back(Child);

        return Child;
    }
}

MicroJSON::Node * MicroJSON::Node::AddObject() {
    if (Type != Types::Array) {
        throw Exception("Cannot add object to array due to type is not array.");
    } else {
        Node *Child = new Node(Types::Object);

        Children.push_back(Child);

        return Child;
    }
}

MicroJSON::Node * MicroJSON::Node::AddArray(const UTF8::String & Name) {
    if (Type != Types::Object) {
        throw Exception("Cannot add array due to type is not parent.");
    } else {
        Node *Child = new Node(Types::Array);

        Child->SetName(Name);

        Children.push_back(Child);

        return Child;
    }
}

MicroJSON::Node * MicroJSON::Node::AddArray() {
    if (Type != Types::Array) {
        throw Exception("Cannot add array to array due to type is not array.");
    } else {
        Node *Child = new Node(Types::Array);

        Children.push_back(Child);

        return Child;
    }
}

void MicroJSON::Node::SetName(const UTF8::String &Name) {
    this->Name = Name;
}

UTF8::String MicroJSON::Node::GetName() const {
    return Name;
}

void MicroJSON::Node::SetType(const int Type) {
    for (int i = 0; i < Children.size(); i++) {
        delete Children[i];
    }
    Children.empty();

    this->Type = Type;
}

void MicroJSON::Node::SetNumber(const long double Number) {
    if (Type != Types::Number) {
        throw Exception("Cannot set number value due to value is not number.");
    } else {
        Type = Types::Number;
        NumberValue = Number;
    }
}

void MicroJSON::Node::SetString(const UTF8::String & Value) {
    if (Type != Types::String) {
        throw Exception("Cannot set string value due to type is not string.");
    } else {

        Type = Types::String;
        StringValue = Value;
    }
}

void MicroJSON::Node::SetBool(const bool &Value) {
    if (Type != Types::Bool) {
        throw Exception("Cannot set bool value due to type is not bool.");
    } else {
        Type = Types::Bool;
        BoolValue = Value;
    }
}