/**
 * MicroJSON library. 
 * 
 * Provides JSON structures parsing and generating in C++ way. 
 * 
 * Distributed under GPL v3 
 * 
 * Author: 
 *      Grigory Gorelov (gorelov@grigory.info)
 *      See more information on grigory.info
 */

#ifndef CNODE_H
#define	CNODE_H

#include <UTF8Strings-1/String.h>
#include <iostream>
#include "Exception.h"

#include <stdlib.h>

namespace MicroJSON {
    /**
     * JSON types enumeration.
     */
    namespace Types {
        const int Object = 1;
        const int Number = 2;
        const int String = 3;
        const int Bool = 4;
        const int Array = 5;
        const int Null = 6;
    }

    class Node {
    public:
        /**
         * Default constructor initializes Node as Object.
         */
        Node();

        /**
         * Constructor initializes Node with given type.
         */
        Node(const int Type);

        /**
         * Copying constructor. Provides creation from another Node object.
         */
        Node(const Node &Original);


        /**
         * Assing operator. Provides Node1=Node2 operation. All data is copied 
         * so Node2 can be destroyed after assignment.
         */
        Node & operator=(const Node &Original);

        
        ~Node();

        /**
         * Change type on the node.
         */
        void SetType(const int Type);

        /**
         * Get current node type.
         */
        int GetType() const;

        /*
         * Wrapper for logical operations. Checks if Node is Object.
         */
        bool IsObject() const;

        /*
         * Wrapper for logical operations. Checks if Node is Number.
         */
        bool IsNumber() const;

        /*
         * Wrapper for logical operations. Checks if Node is String.
         */
        bool IsString() const;

        /*
         * Wrapper for logical operations. Checks if Node is Bool.
         */
        bool IsBool() const;

        /*
         * Wrapper for logical operations. Checks if Node is Array.
         */
        bool IsArray() const;

        /*
         * Wrapper for logical operations. Checks if Node is Null.
         */
        bool IsNull() const;

        /**
         * Get children nodes of the node. Actual for Object and Array nodes.
         */
        std::vector <Node *> GetChildren() const;

        /**
         * Add Number field to Object node.
         * @param Name Field name
         * @param Value Number value
         */
        void AddNumber(const UTF8::String &Name, const double Value);

        /**
         * Add String field to Object node.
         * @param Name Field name
         * @param Value String value
         */
        void AddString(const UTF8::String &Name, const UTF8::String &Value);

        /**
         * Add Bool field to Object node.
         * @param Name Field name
         * @param Value Bool value
         */
        void AddBool(const UTF8::String &Name, const bool &Value);

        /**
         * Add Object field to Object node.
         * @param Name Field name
         * @return Returns pointer to new Node.
         */
        Node * AddObject(const UTF8::String &Name);

        /**
         * Add Array field to Object node.
         * @param Name Field name
         * @return Returns pointer to new Node.
         */
        Node * AddArray(const UTF8::String &Name);

        /**
         * Add Number to Array node.
         */
        void AddNumber(const long Value);

        /**
         * Add String to Array node.
         */
        void AddString(const UTF8::String &Value);

        /**
         * Add Bool to Array node.
         */
        void AddBool(const bool &Value);

        /**
         * Add Object to Array node.
         * @return Returns pointer to new Node.
         */
        Node * AddObject();

        /**
         * Add Array to Array node.
         * @return Returns pointer to new Node.
         */
        Node * AddArray();


        /**
         * Set Number value of Number node.
         */
        void SetNumber(const long double Number);

        /**
         * Get Number value of Number node.
         */
        long double GetNumber() const;

        /**
         * Set String value of String node.
         */
        void SetString(const UTF8::String &Value);

        /**
         * Get String value of String node.
         */
        UTF8::String GetString() const;

        /**
         * Set Bool value of Bool node.
         */
        void SetBool(const bool &Value);

        /**
         * Get Bool value of Bool node.
         */
        bool GetBool() const;

        /**
         * Set field Name of the node.
         */
        void SetName(const UTF8::String &Name);

        /**
         * Get field Name of the node.
         */
        UTF8::String GetName() const;

        /**
         * Parse JSON string and populate current node. Node considered as Root.
         */
        void Parse(const UTF8::String &JSONString);

        /**
         * Get child node with given name of the node.
         */
        Node * GetSubNode(const UTF8::String &Name) const;

        /**
         * Generate JSON string with values of node and subnodes.
         */
        UTF8::String Render() const;

    private:
        UTF8::String Name;
        int Type;

        long double NumberValue;
        UTF8::String StringValue;
        bool BoolValue;

        std::vector <Node *> Children;

        bool _Parse_CharacterIsWhitespace(const UTF8::String &Char);
        void _Parse_ParseObject(Node *Object, const UTF8::String &JSONString, unsigned int &i);
        void _Parse_ParseValue(Node *Object, const UTF8::String &JSONString, unsigned int &i);
        void _Parse_ParseArray(Node *Object, const UTF8::String &JSONString, unsigned int &i);

    };
}

#endif	/* CNODE_H */

